#ifndef s11n_ABSTRACT_CREATOR_H_INCLUDED
#define s11n_ABSTRACT_CREATOR_H_INCLUDED 1

////////////////////////////////////////////////////////////////////////
// abstract_creatore.hpp:
// Defines a difficult-to-describe class. See the docs.
//
//    Author: stephan beal <stephan@s11n.net>
//    License: Public Domain
////////////////////////////////////////////////////////////////////////

#include <string>
#include "classload.hpp" // classloader-related funcs
#include "traits.hpp" // s11n_traits

namespace s11n
{

        /**
           abstract_creator is a helper to avoid some code having to
           know if a type is created on a stack or the heap. This
           makes some template code easier to write, as it avoids
           syntax errors when trying something like:

<pre>
if( object type is a pointer type ) delete(object);
else { ... }
</pre>

           This implementation creates items the stack via the default
           ctor. If instantiated with (T *) a pointer/heap-based
           specialization is activated instead.

           Designed for use with, e.g., ListType<T>, for which T
           may be either T or (T *).

           These objects contain no state information.
        */
        template <typename T>
        struct abstract_creator
        {
                /** Same as (T). */
                typedef T value_type;

                /** Same as (T). */
                typedef T base_value_type;

                /**
                   This implementation assigns v to value_type() and
                   returns true.

                   The string argument is bogus for this implementation,
                   and is used by the pointer specialization to implement
                   polymorphic classloading of value_type.

                   Historical note: before s11n release 0.9.17 this function
                   incorrectly did nothing. While this works for many
                   cases, it caused a subtle, hard-to-track bug in
                   s11n::list::deserialize_list(), in which a deserialized
                   list was re-used when the function was called recursively.
                   Many thanks to Patrick Lin for reporting that problem.

                */
                static bool create( value_type & v,
                                    const std::string & /* implclass */  = std::string()  )
                {
                        // if( 0 ) CERR << "create(&) implclass="<<implclass<<": initializing @" <<std::hex<<&v <<"\n";
                        v = value_type();
                        return true;
                }

                /**
                   Does nothing in this specialization. Specializations
                   which allocate resources are expected to release them here.

                   Note that release() exists because using a static boolean
                   to flag a clean-up mode won't work:

<pre>
typedef double T;
T val;
if( val is a pointer type ) { delete( val ); }
else ...
</pre>

                   That will only work when T is actually a pointer type.
                   Thus the following workaround:

<pre>
typedef [some type] T;
T val;
typedef abstract_creator&lt;T&gt; AC;
assert( AC::create(val) ); // fine for (T) or (T *)
... use val ...
AC::release(val); // deletes pointers. No-op for value types
</pre>
		That works with pointers or non-pointers, and simply does nothing
                for non-pointers.
                 */
                static void release( value_type & )
                {
                        return;
                }

        }; // abstract_creator<T>

        /**
           A specialization of abstract_creator to create objects on
           the heap, using the s11n::cl API.
        */
        template <typename T>
        struct abstract_creator<T *>
        {
                /** Same as (T *). */
                typedef T * value_type;
 
               /** Same as (T). */
                typedef T base_value_type;

                /**
                   Tries to create a value_type object,
                   using classload&lt;base_value_type&gt;( key )
                   to create it. v is assigned to it's value, which
                   may be 0. Returns true if an object is created,
                   else false.

                   The caller owns the returned object.

		   If implclass is NOT passed then it must guess and
		   it is likely to guess incorrectly. It relies on
		   s11n_traits<base_value_type>::class_name(&v)
		   returning a valid value, which is normally only
		   reliable for monomorphic types. It does not use
		   s11n_traits<value_type>::class_name() because
		   base_value_type is the type to which the lib's
		   template-level operations are keyed.

                   Maintenance note: new classloader registrations may
                   need to be installed as new types show up,
                   especially for streamables/PODs (because those
                   aren't normally registered as classes), or this
                   function won't handle them. In s11n this
                   registration is handled by most of the various
                   proxy installation macros. See the headers under
		   proxy/pod_*.hpp for examples.
                */
                static bool create(  value_type & v,
                                     const std::string & implclass = std::string() /* guess! */  )
                {
			typedef s11n_traits<base_value_type> STR;
                        const std::string key = 
                                (! implclass.empty())
                                ? implclass
                                : STR::class_name(&v);
                        v = ::s11n::cl::classload<base_value_type>( key );
                        if( 0 ) CERR << "create(*) implclass=["<<key<<"] allocated @" <<std::hex<<&v <<"\n";
                        return 0 != &v;
                }

                /**
                   Deletes v and assigns it to 0.
                 */
                static void release( value_type & v )
                {
                        //CERR << "release(" << std::hex<<v<<")\n";
                        delete( v );
                        v = 0;
                }
        }; // abstract_creator<T *>


} // namespace

#endif // s11n_ABSTRACT_CREATOR_H_INCLUDED

